;;
;; Mailcrypt
;;
(load-library "mailcrypt") ; provides "mc-setversion"
(mc-setversion "gpg") ;; 2.6 for PGP 2.6, 5.0 for PGP 5.0

(autoload 'mc-install-write-mode "mailcrypt" nil t)
(autoload 'mc-install-read-mode "mailcrypt" nil t)

(add-hook 'mail-mode-hook 'mc-install-write-mode)
(add-hook 'gnus-summary-mode-hook 'mc-install-read-mode)
(add-hook 'message-mode-hook 'mc-install-write-mode)
(add-hook 'news-reply-mode-hook 'mc-install-write-mode)

;; USER SPECIFIC OPTIONS

;; this is my normal identity to use:
(setq mc-pgp-user-id "greve@gnu.org")
(setq mc-gpg-user-id "greve@gnu.org")

;; timeouts
(setq mc-passwd-timeout 2400)

;; do we always want to sign encrypted messages? you bet!
(setq mc-pgp-always-sign t)

;; pop up the GPG output window when snarfing keys.
(setq mc-gpg-display-snarf-output t)

;; Assume that the message should be encoded for 
;; everyone listed in the To, Cc, and Bcc fields.
(setq mc-use-default-recipients t)

;; Encrypt all outgoing messages with user's public key.
;; since I want to be able to read it, this seems like a
;; good idea... ,-)
(setq mc-encrypt-for-me t)

;; MML:
(setq mml2015-use 'mailcrypt)
(setq mm-verify-option 'known)
(setq mm-decrypt-option 'always)

;; gpg-ring.el
(require 'gpg-ring)

;;
;; cludge for the Mailcrypt/gnus interoperability insufficiency:
;; replacement for function to get recipients from gnus in CVS
;; Copyright (C) 1996, 1997, 1998, 1999, 2000
;;        Free Software Foundation, Inc.
;; Copyright (C) 2000 Georg C. F. Greve
;;
(defun message-options-set-recipient ()
  
  ;;
  ;; function to get default recipients from mailcrypt 3.5.5
  ;; Copyright (C) 1995  Jin Choi <jsc@mit.edu>
  ;;                     Patrick LoPresti <patl@lcs.mit.edu>
  ;; licensed under the GNU General Public License
  ;; --begin--
  (setq default-recipients
	(save-restriction
	  (goto-char (point-min))
	  (re-search-forward
	   (concat "^" (regexp-quote mail-header-separator) "$"))
	  (narrow-to-region (point-min) (point))
	  (and (featurep 'mailalias)
	       (not (featurep 'mail-abbrevs))
	       mail-aliases
	       (expand-mail-aliases (point-min) (point-max)))
	  (mc-strip-addresses
	   (mapcar 'cdr
		   (mc-get-fields "to\\|cc\\|bcc")))))
  ;; --end--

  ;; find crypto tag and extract possible recipients
  (setq crypto-tag (gnus-look-for-pgp-crypto-tag))
  (message-goto-body)

  ;; if crypto-tag exists: what about specified recipients?
  (if (and (setq p (search-forward "recipients=\"" crypto-tag t)) crypto-tag)
      (setq default-recipients 
	    (buffer-substring p (- (search-forward "\"" crypto-tag t) 1))))
 
  ;; slightly modified original function
  (save-restriction
    (message-narrow-to-headers-or-head)
    (message-options-set 'message-sender
                         (mail-strip-quoted-names 
                          (message-fetch-field "from")))
    (message-options-set 'message-recipients
			 (mail-strip-quoted-names 
			  (if crypto-tag
			      (read-string "Recipients: " default-recipients )
			    (message-fetch-field "to"))))))

;;
;; some functions for nicer crypto usability in gnus
;;
;; Copyright (C) 2000 Georg C. F. Greve greve@gnu.org
;; license: GNU General Public License
;;

(defun gnus-look-for-pgp-tag ()
  "Checks whether this mail does have any PGP tags set.  
If the first line of the mail begins with something that 
appears to be a MIME crypto tag, it returns the position
of the end of that line. nil if no tag could be found."
  (interactive)
  (message-goto-body)
  (setq p (point-marker))
  (search-forward "=" nil 2)
  (if (or 
       (string= "<#part sign=" (buffer-substring p (point)))
       (string= "<#part encrypt=" (buffer-substring p (point)))
       (string= "<#multipart sign=" (buffer-substring p (point)))
       (string= "<#multipart encrypt=" (buffer-substring p (point))))
      (search-forward "\n")
    nil))

(defun gnus-look-for-pgp-crypto-tag ()
  "Checks whether this mail does have any PGP crypto tags 
set. If the first line of the mail begins with something that 
appears to be a MIME crypto tag, it returns the position
of the end of that line. nil if no tag could be found."
  (interactive)
  (message-goto-body)
  (setq p (point-marker))
  (search-forward "=" nil 2)
  (if (or 
       (string= "<#part encrypt=" (buffer-substring p (point)))
       (string= "<#multipart encrypt=" (buffer-substring p (point))))
      (search-forward "\n") 
    nil))

(defun gnus-set-pgp-none ()
  "Removes the tag for PGP signing or encrypting if there is one."
  (interactive)
  (message-goto-body)
  (setq p (point-marker))
  (if (setq q (gnus-look-for-pgp-tag))
      (delete-region p q))
  (goto-char p))

(defun gnus-set-pgp-sign ()
  "Inserts the tag for PGP signature after removing old tag (if necessary)."
  (interactive)
  (gnus-set-pgp-none)
  (if (not (bolp))
      (insert ?\n))
  (insert (format "<#multipart sign=pgpmime>\n")))

(defun gnus-set-pgp-encrypt ()
  "Inserts the tag for PGP encryption after removing old tag (if necessary)."
  (interactive)
  (gnus-set-pgp-none)
  (if (not (bolp))
      (insert ?\n))
  (insert (format "<#multipart encrypt=pgpmime>\n")))

(defun gnus-set-pgp-encrypt-with-recipients (&optional default-recipients)
  "Removes the old tag (if necessary) and then asks for the recipients of the 
encrypted mail. If those are non-empty, a new PGP encryption tag is inserted."
  (interactive)
  (gnus-set-pgp-none)
  (setq recipients 
	 (read-from-minibuffer "Recipients: " 
			       (if default-recipients
				   default-recipients
				 ;; function to get default recipients from mailcrypt 3.5.5
				 ;;   mc-toplev.el, entry point functions for Mailcrypt
				 ;;   Copyright (C) 1995  Jin Choi <jsc@mit.edu>
				 ;;                       Patrick LoPresti <patl@lcs.mit.edu>
				 ;;   license: GNU General Public License
				 (save-restriction
				   (goto-char (point-min))
				   (re-search-forward
				    (concat "^" (regexp-quote mail-header-separator) "$"))
				   (narrow-to-region (point-min) (point))
				   (and (featurep 'mailalias)
					(not (featurep 'mail-abbrevs))
					mail-aliases
					(expand-mail-aliases (point-min) (point-max)))
				   (mc-strip-addresses
				    (mapcar 'cdr
					    (mc-get-fields "to\\|cc\\|bcc")))))))
  (message-goto-body)
  (if (not (string= recipients ""))
      (insert (format "<#multipart encrypt=pgpmime recipients=\"%s\">\n" recipients))
    nil ))

(defun gnus-set-pgp-encrypt-if-nothing (&optional ask) 
  "Inserts the tag for encryption via PGP if nothing else is set."
  (interactive)
  (if (not (gnus-look-for-pgp-tag))
      (if (if ask (y-or-n-p "Should this mail be encrypted?") t )
	  (gnus-set-pgp-encrypt))))

(defun gnus-set-pgp-sign-if-nothing (&optional ask) 
  "Inserts the tag for signing via PGP if nothing else is set. 
Optionally asks whether this is desired."
  (interactive)
  (if (not (gnus-look-for-pgp-tag))
      (if (if ask (y-or-n-p "Should this mail be signed?") t )
	  (gnus-set-pgp-sign))))

(defun gnus-set-pgp-sign-interactively ()
  "Inserts the PGP signing tag after asking. To be used mainly for hook fun.
Asks for encryption if signing is declined."
  (interactive)
  (gnus-set-pgp-sign-if-nothing t)
  (gnus-set-pgp-encrypt-if-nothing t))

(defun gnus-insert-kill-as-mime-public-key ()
  "Inserts a public key from the kill buffer as MIME part in the current mail."
  (interactive)
  (let ((oldbuf (current-buffer)))
    (save-current-buffer
      (set-buffer 
       (setq keybuf (buffer-name (generate-new-buffer "*Public-PGP-Key-Data*"))))
      
      (yank)

      ))
  (if (not (bolp))
      (insert ?\n))
  (mml-attach-buffer keybuf "application/pgp-keys"))

;;
;; pgp-mail field support for BBDB
;;
;; from bbdb-gpg.el by Kevin Davidson <tkld@quadstone.com>
;; Copyright (C) 1997 Kevin Davidson
;; Copyright (C) 2000 Georg C. F. Greve <greve@gnu.org>
;; license: GNU General Public License
;;
(require 'message)
(require 'bbdb)
(require 'mailcrypt)

(defconst bbdb-pgp-version (substring "$Revision: 1.3P $" 11 -2)
  "$Id: bbdb-pgp.el,v 1.3 1997/11/10 15:20:29 tkld Exp $
Patched by: Georg C. F. Greve greve@gnu.org
Report bugs to: Kevin Davidson tkld@quadstone.com")

(defvar bbdb-pgp-field 'pgp-mail
  "*Field to use in BBDB to store PGP preferences.
If the value is \"encrypt\" then messages are encrypted.
If the value is \"sign\" then messages are signed.")

(defun bbdb/pgp-get-pgp (name address)
  "Look up user NAME and ADDRESS in BBDB and return the PGP preference."
  (let* ((record (bbdb-search-simple name address))
	 (pgp (and record
		   (bbdb-record-getprop record bbdb-pgp-field))))
    pgp))

(defun bbdb/pgp-hook-fun ()
  "Function to be added to message-send-hook
Uses PGP to encrypt messages to users marked in the BBDB with the
field bbdb-pgp-field."
  (save-restriction
    (save-excursion
      (message-narrow-to-headers)
      (let* ((to-field (mail-fetch-field "To" nil t))
	     (address (mail-extract-address-components (or to-field ""))))
	(widen)
	(if (not (equal address '(nil nil)))
	    (let ((pgp-p (bbdb/pgp-get-pgp (car address) (car (cdr address)))))
	      (cond
	       ((string= "encrypt" pgp-p) 
		(gnus-set-pgp-encrypt-if-nothing))
	       ((string= "sign" pgp-p)
		(gnus-set-pgp-sign-if-nothing))
	       (t nil))))))))



;;
;; first pay attention to the BBDB field and then ask me
;; if there is still nothing set as I probably forgot
;;

(add-hook 'message-send-hook 'gnus-set-pgp-sign-interactively)
(add-hook 'mail-send-hook 'gnus-set-pgp-sign-interactively)

(add-hook 'message-send-hook 'bbdb/pgp-hook-fun)
(add-hook 'mail-send-hook 'bbdb/pgp-hook-fun)
